from typing import List, Optional, Union
from datalogue.clients._http import _HttpClient, HttpMethod
from datalogue.models.job import Job, JobStatus, JobType
from datalogue.dtl_utils import _parse_list
from datalogue.errors import DtlError
from datetime import datetime
from uuid import UUID


class JobsClient:
    """
    Client to interact with the Scheduled pipelines
    """

    def __init__(self, http_client: _HttpClient):
        self.http_client = http_client
        self.service_uri = "/scout"

    def list(
        self,
        statuses: List["JobStatus"] = [
            JobStatus.Scheduled,
            JobStatus.Defined,
            JobStatus.Running,
            JobStatus.Succeeded,
            JobStatus.Failed,
            JobStatus.Unknown,
            JobStatus.Cancelled,
        ],
        job_types: List["JobType"] = [JobType.Pipeline, JobType.Sampling],
        page: int = 1,
        item_per_page: int = 25,
        pipeline_ids: Optional[List[UUID]] = None,
        scheduled_from: Optional[datetime] = None,
        scheduled_until: Optional[datetime] = None,
        job_duration_min: Optional[int] = None,
        job_duration_max: Optional[int] = None,
        job_owners: Optional[List[UUID]] = None,
        connection_ids: Optional[List[UUID]] = None,
    ) -> Union[DtlError, List[Job]]:
        """
        List jobs

        :param page: page to be retrieved
        :param item_per_page: number of jobs to be put in a page
        :param pipeline_id: optional pipeline id used to retrieve related jobs.
        :return: Returns a List of all the available Jobs or an error message as a string
        """

        statuses_list = [s.value for s in statuses]
        job_type_list = [jt.value for jt in job_types]

        endpoint = f"/v2/jobs?page={page}&size={item_per_page}"

        params = {"job-statuses": statuses_list, "job-type-filter": job_type_list}

        if pipeline_ids:
            params["stream-ids"] = pipeline_ids
        if job_duration_max:
            params["duration-max"] = job_duration_max
        if job_duration_min:
            params["duration-min"] = job_duration_min
        if scheduled_from:
            params["scheduled-from"] = created_from.strftime("%Y-%m-%dT%H:%M:%SZ")
        if scheduled_until:
            params["scheduled-until"] = created_until.strftime("%Y-%m-%dT%H:%M:%SZ")
        if job_owners:
            params["owner"] = job_owners
        if connection_ids:
            params["connection-ids"] = connection_ids

        res = self.http_client.make_authed_request(
            path=self.service_uri + endpoint, method=HttpMethod.GET, params=params
        )

        if isinstance(res, DtlError):
            return res

        return _parse_list(Job._from_payload)(res)

    def cancel(self, job_id: UUID) -> Union[DtlError, Job]:
        """"
        Cancel a Job given job_id.

        :param job_id:
        :return: Returns the Job object canceled or an error
        """

        res = self.http_client.make_authed_request(self.service_uri + f"/jobs/{str(job_id)}/cancel", HttpMethod.POST)

        if isinstance(res, DtlError):
            return res

        return Job._from_payload(res)

    def get(self, job_id: UUID) -> Union[DtlError, Job]:
        """"
        Get a Job given job_id.

        :param job_id:
        :return: Returns the Job object or an error
        """
        endpoint = f"/v2/jobs"

        statuses = JobStatus.values()
        job_types = JobType.values()

        params = {"job-statuses": statuses, "job-type-filter": job_types, "id": job_id, "page": 1, "size": 1}
        res = self.http_client.make_authed_request(
            path=self.service_uri + endpoint, method=HttpMethod.GET, params=params
        )
        if isinstance(res, DtlError):
            return res

        if not isinstance(res, list):
            return DtlError("Response from server should be a list")

        if len(res) < 1:
            return DtlError(f"No job results found with id {job_id}")

        return Job._from_payload(res[0])



from datetime import datetime
from uuid import UUID
from typing import Union, Optional, List
from dateutil.parser import parse
from datalogue.errors import DtlError


class DataProduct:
    """
    Data Product is a collection of multiple pipelines, streaming from a user defined set of sources
    to a user defined set of destinations, creating a data set to service a specific use case.
    Grouping pipelines into use-case specific Data Products allows for clear ownership,
    governance as well as easy updating to ensure the end data set is fresh, trustable and reliable.

    :param name: name of the Data Product (name must be unique)
    :param description: description of your Data Product (Optional)
    :param pipeline_ids: list of pipeline ids (pipeline_id and Data Product mapping is unique)
    :id: unique uuid of the Data Product, auto-generated by backend
    :owner: represents the uuid of a user, who owns the Data Product, auto-generated by backend
    :created_by: uuid of the creator of the Data Product, auto-generated by backend
    :created_at: timestamp at which the Data Product is created, auto-generated by backend
    :last_updated_by: uuid of the user, who last modified the Data Product, auto-generated by backend
    :last_updated_at: timestamp at which the Data Product was last modified, auto-generated by backend
    """

    def __init__(
        self,
        name: str,
        description: Optional[str] = None,
        pipeline_ids: List[UUID] = [],
        id: Optional[UUID] = None,
        owner: Optional[UUID] = None,
        created_by: Optional[UUID] = None,
        created_at: Optional[datetime] = None,
        last_updated_by: Optional[UUID] = None,
        last_updated_at: Optional[datetime] = None,
    ):
        self.id = id
        self.name = name
        self.description = description
        self.owner = owner
        self.created_by = created_by
        self.created_at = created_at
        self.last_updated_by = last_updated_by
        self.last_updated_at = last_updated_at
        self.pipeline_ids = pipeline_ids

    def __eq__(self, other):
        return (
            (self.id == other.id)
            and (self.name == other.name)
            and (self.created_at == other.created_at)
            and (self.created_by == other.created_by)
        )

    def __repr__(self):
        """
        Represents a Data Product Object in a user friendly readable format.
        """
        return (
            f"{self.__class__.__name__}(\n "
            f"id= {self.id!r},\n "
            f"name= {self.name!r},\n "
            f"description= {self.description!r},\n "
            f"pipeline_ids= {self.pipeline_ids!r},\n "
            f"owner= {self.owner!r},\n "
            f"created_by= {self.created_by!r},\n "
            f"created_at= {self.created_at!r},\n "
            f"last_updated_by= {self.last_updated_by!r},\n "
            f"last_updated_at= {self.last_updated_at!r}\n "
            ")"
        )

    def _as_payload(self) -> dict:
        """
        Parses a Data Product object into a dictionary.
        """
        payload = {"name": self.name}
        if self.description is not None:
            payload["description"] = self.description
        payload["streamIds"] = self.pipeline_ids
        return payload

    @classmethod
    def _from_payload(cls, json: dict) -> Union[DtlError, "DataProduct"]:
        """
        Parses a dictionary into a Data Product object.
        """
        id = json.get("id")
        if id is None:
            return DtlError("Data Product object should have a 'id' property")
        else:
            try:
                id = UUID(id)
            except ValueError:
                return DtlError("'id' field was not a proper uuid")

        name = json.get("name")
        if name is None:
            return DtlError("Data Product object should have a 'name' property")

        description = json.get("description")
        owner = json.get("owner")
        if owner is None:
            return DtlError("Data Product object should have a 'owner' property")
        else:
            try:
                owner = UUID(owner)
            except ValueError:
                return DtlError("'owner' field was not a proper uuid")

        created_by = json.get("createdBy")
        if created_by is None:
            return DtlError("Data Product object should have a 'createdBy' property")
        else:
            try:
                created_by = UUID(created_by)
            except ValueError:
                return DtlError("'createdBy' field was not a proper uuid")

        created_at = json.get("createdAt")
        if created_at is None:
            return DtlError("Data Product object should have a 'createdAt' property")
        else:
            try:
                created_at = parse(created_at)
            except ValueError:
                return DtlError("The 'createdAt' could not be parsed as a valid date")

        last_updated_by = json.get("lastUpdatedBy")
        if last_updated_by is None:
            return DtlError("Data Product object should have a 'lastUpdatedBy' property")
        else:
            try:
                last_updated_by = UUID(last_updated_by)
            except ValueError:
                return DtlError("'lastUpdatedBy' field was not a proper uuid")

        last_updated_at = json.get("lastUpdatedAt")
        if last_updated_at is None:
            return DtlError("Data Product object should have a 'lastUpdatedAt' property")
        else:
            try:
                last_updated_at = parse(last_updated_at)
            except ValueError:
                return DtlError("The 'lastUpdatedAt' could not be parsed as a valid date")

        pipeline_ids = json.get("streamIds")
        if pipeline_ids is None:
            return DtlError("Data Product object should have a 'streamIds' property")

        return cls(
            name=name,
            id=id,
            description=description,
            owner=owner,
            created_by=created_by,
            created_at=created_at,
            last_updated_by=last_updated_by,
            last_updated_at=last_updated_at,
            pipeline_ids=pipeline_ids,
        )



from typing import List, Union, Optional

from datalogue.clients._http import _HttpClient, HttpMethod
from datalogue.models.datastore import Datastore, CredentialType
from datalogue.models.credentials import (
    Credentials,
    _credentials_ref_from_payload,
    CredentialsReference,
)
from datalogue.dtl_utils import _parse_list
from uuid import UUID
from datalogue.errors import DtlError
from datalogue.models.permission import CredentialPermission


class CredentialsClient:
    """
    Client to interact with the Credentials objects
    """

    def __init__(self, http_client: _HttpClient):
        self.http_client = http_client
        self.service_uri = "/scout"

    def list(
        self, credential_type: Optional[str] = None, page: int = 1, item_per_page: int = 25,
    ) -> Union[DtlError, List[Datastore]]:
        """
        Lists available credentials that user has Use Permission

        :param credential_type: type of the credentials (S3, GCS, Jdbc, etc)
        :param page: page to be retrieved
        :param item_per_page: number of items to be put in a page
        :return: Returns a List of all the available credentials or an error message as a string
        """

        res = self.http_client.make_authed_request(
            self.service_uri
            + f"/credentials?page={page}&size={item_per_page}{'' if credential_type is None else f'&type={credential_type}'}",
            HttpMethod.GET,
        )

        if isinstance(res, DtlError):
            return res

        return _parse_list(_credentials_ref_from_payload)(res)

    def create(
        self,
        credentials_definition: Credentials,
        name: Optional[str] = None,
        is_destination: Optional[bool] = False,
        data_models: [UUID] = [],
        load_schema_and_samples: Optional[bool] = False,
    ) -> Union[DtlError, CredentialsReference]:
        """
        Creates the Credentials as specified.

        :param credentials_definition: Credentials Definition to be creates
        :param name: Name to be given to the Credentials to be able to find it later
        :param is_destination: Boolean to specify if credential refers to a destination or not.
        :param data_models: List of data model UUIDs that are related to the data in the specified credentials.
        :param load_schema_and_samples: If True, the sampling and schema extraction will be performed automatically for
         all datastores of this credential (False by default) (temporarily disabled)
        :return: string with error message if failed, uuid otherwise
        """
        load_schema_and_samples = False

        payload = credentials_definition._as_payload()

        if credentials_definition.type == CredentialType.S3:
            if load_schema_and_samples is True and "connectionBucket" not in payload:
                return DtlError("`connection_bucket` must be supplied, when `load_schema_and_samples` is True")

        res = self.http_client.make_authed_request(
            self.service_uri + "/credentials",
            HttpMethod.POST,
            {
                "name": name,
                "credentials": payload,
                "isDestination": is_destination,
                "dataModels": data_models,
                "loadSchemaAndSamples": load_schema_and_samples,
            },
        )

        if isinstance(res, DtlError):
            return res

        return _credentials_ref_from_payload(res)

    def update(
        self, credentials_id: UUID, credentials_definition: Credentials, name: Optional[str] = None,
    ) -> Union[DtlError, CredentialsReference]:
        """
        Updates the given Credentials id with name and credentials

        TODO rework the api so that you can only update the name without the Credentials themselves

        :param credentials_id: id of te Credentials to update
        :param name: if specified, will overwrite current name
        :param credentials_definition: new Credentials definition
        :return:
        """

        body = {}
        if name is not None:
            body["name"] = name

        body["credentials"] = credentials_definition._as_payload()

        res = self.http_client.make_authed_request(
            self.service_uri + "/credentials/" + str(credentials_id), HttpMethod.PUT, body,
        )

        if isinstance(res, DtlError):
            return res

        return _credentials_ref_from_payload(res)

    def delete(self, credentials_id: UUID) -> Union[DtlError, bool]:
        """
        Deletes the given Credentials

        :param credentials_id: id of the credentials to be deleted
        :return: true if successful, false otherwise
        """

        res = self.http_client.make_authed_request(
            self.service_uri + "/credentials/" + str(credentials_id), HttpMethod.DELETE
        )

        if isinstance(res, DtlError):
            return res
        else:
            return True

    def share(self, credentials_id: UUID, group_id: UUID, permission: CredentialPermission) -> Union[DtlError, bool]:
        """
        Shares the given Credentials

        :param credentials_id: id of the credentials to be shared
        :param group_id: id of the group to be shared with
        :param permission: level of the Permission
        :return: true if successful, false otherwise
        """
        res = self.http_client.make_authed_request(
            self.service_uri
            + f"/credentials/{str(credentials_id)}/permissions/groups/{str(group_id)}/{str(permission.value)}",
            HttpMethod.PUT,
        )

        if isinstance(res, DtlError):
            return res

        return True

    def validate(
        self, credentials_id: Optional[Union[str, UUID]] = None, credentials_definition: Optional[Credentials] = None,
    ) -> Union[bool, DtlError]:
        """
        Validate that credentials can access data; usable with created credentials or a local Credentials object

        :param credentials_id: id of the credentials you wish to validate
        :param credentials_definition: local definition of a Credentials object; for pre-creation validation
        :return: True if successful, or DtlError if failed
        """

        if credentials_id is not None and credentials_definition is not None:
            return DtlError(
                "Both credentials_id & credentials_definition cannot be specified together for validating credentials"
            )

        if credentials_id is not None:
            res = self.http_client.make_authed_request(
                self.service_uri + f"/credentials/{str(credentials_id)}/test", HttpMethod.POST,
            )
        elif credentials_definition is not None:
            res = self.http_client.make_authed_request(
                self.service_uri + "/credentials/test",
                HttpMethod.POST,
                {"credentials": credentials_definition._as_payload(), "name": ""},
            )

        else:
            res = DtlError("Either credentials_id or credentials_definition must be supplied to validate credentials")

        if isinstance(res, DtlError):
            return res

        return True



from typing import *
from typing import BinaryIO
from enum import Enum
import os

from datalogue.dtl_utils import Json
from datalogue.errors import DtlError
import logging
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from requests import Response
from datalogue.models.organization import _users_from_payload
from pathlib import Path

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

logger = logging.getLogger("_HttpClient")


class HttpMethod(Enum):
    POST = "POST"
    GET = "GET"
    PUT = "PUT"
    DELETE = "DELETE"
    PATCH = "PATCH"


class MultiPartFile:
    def __init__(self, name: str, file: BinaryIO):
        self.name = name
        self.file = file


class MultiPartData:
    def __init__(self, name: str, data: str):
        self.name = name
        self.data = data


class _HttpClient:
    """
    Class that Abstracts aways requests and makes it use type safe.
    It also prepares all the requests to be used authenticated to the API
    """

    def __init__(
        self,
        uri: str,
        authentication_name: Optional[str] = None,
        verify_certificate: bool = True,
        encoded_auth_header: Optional[str] = None,
    ):
        self.uri = uri
        self.http_session = requests.Session()
        self.verify_certificate = verify_certificate
        self.authentication_name = authentication_name
        if not verify_certificate:
            logger.info("SSL Certification verification is disabled.")
        if encoded_auth_header is not None:
            self.http_session.headers.update({"Authorization": encoded_auth_header})

    def get_csrf(self) -> Response:
        public_url = self.uri
        if os.environ.get("DTL_PUBLIC_URL") is not None:
            public_url = os.environ.get("DTL_PUBLIC_URL")

        self.http_session.headers.update({"Origin": public_url.strip("/api")})

        url = self.uri + "/health"
        res = self.http_session.get(url, verify=self.verify_certificate)

        if "csrf-token" in res.headers:
            self.http_session.cookies.set("csrf-token", res.headers["csrf-token"])
            self.http_session.headers.update({"X-Csrf-Token": res.headers["csrf-token"]})
        return res

    def login(self, username: str, password: str) -> Union[DtlError, None]:
        """
        Sets the cookie for the client to use

        :param username: Username of user to login
        :param password: Password of user to login
        :return: Nothing
        """
        self.get_csrf()

        url = self.uri + "/signin"
        try:
            res = self.http_session.post(
                url,
                json={
                    "emailOrUsername": username,
                    "password": password,
                    "authenticationMethodName": self.authentication_name,
                    "clientType": "sdk",
                },
                allow_redirects=False,
                verify=self.verify_certificate,
            )
        except requests.ConnectionError:
            return DtlError("There was a connection error trying to reach: %s" % self.uri)
        except requests.HTTPError:
            return DtlError("There was an HTTP error trying to reach: %s" % self.uri)
        except requests.URLRequired:
            return DtlError("%s is not a valid endpoint." % self.uri)
        except requests.Timeout:
            return DtlError("The request timed out")

        if res.status_code != 201 and res.status_code != 302 and res.status_code != 200:
            return DtlError("Authentication failed - %s : %s" % (str(res.status_code), res.text))
        elif res.status_code == 503 or res.status_code == 504:
            return DtlError("The service is not available. StatusCode: %s, Body: %s" % (str(res.status_code), res.text))
        else:
            return None

    def signup(self, first_name, last_name, email, password, accept_terms):
        """
        Allows
        :param first_name: First name of user to be registered
        :param last_name: Last name of user to be registered
        :param email: Email of user to be registered
        :param password: Password of user to be registered
        :param accept_terms: Whether user accepts the term of service here : https://assets.website-files.com/5c980fa2de94e8f79f047ded/5cd0e4d60880b51cb361ae7a_Terms%20of%20Service.pdf
        :return: A User object
        """
        print(
            "By calling this action, you've agreed to the terms of service as set forth here : "
            "https://assets.website-files.com/5c980fa2de94e8f79f047ded/5cd0e4d60880b51cb361ae7a_Terms%20of%20Service.pdf"
        )

        if not accept_terms:
            return DtlError("Terms & Conditions for registration are not accepted")
        url = self.uri + "/signup"
        self.get_csrf()

        try:
            res = self.http_session.post(
                url,
                json={"email": email, "firstName": first_name, "lastName": last_name, "password": password,},
                verify=self.verify_certificate,
            )
        except requests.ConnectionError:
            return DtlError("There was a connection error trying to reach: %s" % self.uri)
        except requests.HTTPError:
            return DtlError("There was an HTTP error trying to reach: %s" % self.uri)
        except requests.URLRequired:
            return DtlError("%s is not a valid endpoint." % self.uri)
        except requests.Timeout:
            return DtlError("The request timed out")

        if res.status_code == 200:
            if res.headers.get("content-type") == "application/json":
                return _users_from_payload(res.json())
            else:
                return DtlError("Cannot parse response, unknown type")
        elif res.status_code == 503:
            return DtlError("The service is not available")
        elif res.status_code == 504:
            return DtlError("The service is not available: Gateway Timeout!")
        else:
            return DtlError(res.text)

    @staticmethod
    def _handle_status_code(request_uri: str, res: Response) -> Union[DtlError, None]:
        """
        Return a proper DtlError depending on the status code of the Response

        :param request_uri: uri of th request that was made
        :param res: Response received to the request
        :return: an Error or None if the status doesn't warrant returning an error
        """
        if res.status_code == 400:
            return DtlError("Invalid Request to %s returned 400, body res: %s" % (request_uri, res.text))

        if res.status_code == 401:
            return DtlError("It looks like you are not logged in, please login")

        if res.status_code == 403:
            return DtlError(f"Authorization error. Make sure you're authorized to do this operation. Msg: {res.text}")

        if res.status_code == 404:
            return DtlError("Route not found %s" % request_uri)

        if res.status_code == 413:
            return DtlError("Body is too big, please limit uploads to 1 Go")

        if res.status_code == 500:
            if not res.text:
                return DtlError("Internal Server error")
            else:
                return DtlError(res.text)

        if res.status_code == 503:
            return DtlError("The service is not available")

        if res.status_code == 504:
            return DtlError("The service is not available: Gateway timeout!")

        if res.status_code == 307:
            return DtlError(
                "You have been logged out due to a backend restart, probably due to an update or other maintenance. Please log in again."
            )

        return None

    def post_multipart_data(
        self, path: str, files: List[MultiPartFile], data: List[MultiPartData]
    ) -> Union[DtlError, Response]:
        headers = {"content-type": "multipart/form-data"}

        files_payload = {}
        for f in files:
            files_payload[f.name] = f.file

        data_payload = {}
        for d in data:
            data_payload[d.name] = d.data

        request_uri = self.uri + path
        req = requests.Request(HttpMethod.POST.value, request_uri, files=files_payload, data=data_payload)
        prepped = self.http_session.prepare_request(req)
        res = self.http_session.send(prepped, verify=self.verify_certificate)

        e = self._handle_status_code(request_uri, res)
        if isinstance(e, DtlError):
            return e

        content_type_header = res.headers.get("content-type")
        if len(res.text) > 0 and content_type_header == "application/json":
            return res.json()
        else:
            return res.text

    def post_binary(self, path: str, file: BinaryIO, params: Optional[dict] = None) -> Union[DtlError, Response]:
        """
        Makes a post request with the content being a binary file that will be uploaded chunk by chunk

        :param path: path uri to the datastore
        :param file: iterator that is going to generate the binary data
        :param params: parameters to be used in the request
        :return:
        """
        request_uri = self.uri + path
        req = requests.Request(HttpMethod.POST.value, request_uri, data=file, params=params)
        prepped = self.http_session.prepare_request(req)

        prepped.headers["Content-Type"] = "application/octet-stream"

        res = self.http_session.send(prepped, verify=self.verify_certificate)

        e = _HttpClient._handle_status_code(request_uri, res)
        if isinstance(e, DtlError):
            return e

        return res

    def make_authed_request(
        self, path: str, method: HttpMethod, body: Optional[dict] = None, params: Optional[dict] = None, stream=False,
    ) -> Union[DtlError, Json, Response]:
        res = self.execute_authed_request(path, method, body, params, stream)
        request_uri = self.uri + path

        e = self._handle_status_code(request_uri, res)
        if isinstance(e, DtlError):
            return e

        content_type_header = res.headers.get("content-type")
        if stream:
            return res
        elif len(res.text) > 0 and content_type_header == "application/json":
            return res.json()
        else:
            return res.text

    def execute_authed_request(
        self, path: str, method: HttpMethod, body: Optional[dict] = None, params: Optional[dict] = None, stream=False,
    ) -> Union[DtlError, Response]:
        """
        Makes a request to the API give the url

        :param path: path to access the datastore below the host
        :param method: parameter to indicate HTTP Method to be used in the request
        :param body: body to be sent along with the request
        :param params: query parameters, to be fed with a dictionary
        :param stream: boolean to activate streaming the content of the response, this will delay the retrieval of
        the body until .content is called.
        :return: Request response in raw format
        """
        csrf_res = self.get_csrf()
        if csrf_res.status_code == 307:
            e = self._handle_status_code(request_uri, csrf_res)
            if isinstance(e, DtlError):
                return e

        request_uri = self.uri + path
        req = requests.Request(method.value, request_uri, json=body, params=params)
        prepped = self.http_session.prepare_request(req)
        res = self.http_session.send(prepped, stream=stream, verify=self.verify_certificate)
        return res

    def put_binary(self, path: str, file: BinaryIO, params: Optional[dict] = None) -> Union[DtlError, Response]:
        """
        Makes a put request with the content being a binary file that will be uploaded chunk by chunk

        :param path: path uri to the datastore
        :param file: iterator that is going to generate the binary data
        :param params: parameters to be used in the request
        :return:
        """
        csrf_res = self.get_csrf()
        if csrf_res.status_code == 307:
            return csrf_res
        request_uri = self.uri + path
        req = requests.Request(HttpMethod.PUT.value, request_uri, data=file, params=params)
        prepped = self.http_session.prepare_request(req)

        prepped.headers["Content-Type"] = "application/octet-stream"

        res = self.http_session.send(prepped, verify=self.verify_certificate)

        e = _HttpClient._handle_status_code(request_uri, res)
        if isinstance(e, DtlError):
            return e

        return res



# check the input here: https://github.com/datalogue/scout/blob/master/src/main/scala/io/datalogue/scout/api/inputs.scala
from typing import Dict, List, Optional, Union
from datalogue.errors import DtlError
from datalogue.models.datastore import Datastore, _datastore_from_payload
from datalogue.dtl_utils import _parse_list, _parse_uuid
from uuid import UUID


class DatastoreCollection:
    def __init__(
        self,
        name: str,
        store_ids: List[str],
        stores: Optional[List[Datastore]] = None,
        description: str = None,
        tags: List[str] = [],
        ontology_ids: List[Union[str, UUID]] = [],
        id: Optional[UUID] = None,
    ):
        """
        Build a local datastore collection containing a list of datastores for simple viewing and management
        :param name: a name for your datastore collection
        :param store_ids: a list of ids referencing the datastores to be collected in the datastore collection
        :param stores: once created, this property will contain a list of the datastore objects thus collected, for easy iteration
        :param description: a description for your datastore collection,
        :param tags: can be used to apply string tags to your datastore collection
        :param ontology_ids: can be used to map this datastore collection to an ontolog, for the suggestion engine flow accessible on the GUI
        :param id: once creared, this property will contain the unique id
        """
        self.name = name
        self.store_ids = store_ids
        self.stores = stores
        self.id = id
        self.description = description
        self.tags = tags
        self.ontology_ids = ontology_ids
        self._as_payload()

    def __eq__(self, other: "DatastoreCollection"):
        if isinstance(self, other.__class__):
            return self._as_payload() == other._as_payload()
        return False

    def __repr__(self):
        return (
            f"{self.__class__.__name__}("
            f"id= {self.id}, "
            f"name= {self.name!r}, "
            f"description= {self.description!r}, "
            f"store_ids= {self.store_ids!r}, "
            f"ontology_ids= {self.ontology_ids!r}, "
            f"tags= {self.tags!r})"
        )

    def _as_payload(self) -> dict:
        """
        Dictionary representation of the object
        :return:
        """
        base = {
            "name": self.name,
            "storeIds": self.store_ids,
            "description": self.description,
            "tags": self.tags,
            "ontologyIds": list(map(lambda u: str(u), self.ontology_ids)),
        }

        # toDo unpack proper these values
        if self.id is not None:
            base["id"] = str(self.id)
        return base


def _datastore_collection_from_dict(json: dict) -> Union[DtlError, DatastoreCollection]:
    """Datastore Collection instance from dict."""
    name = json.get("name")
    if name is None:
        return DtlError("'name' for a datastore collection should be defined")

    id = json.get("id")
    if id is None:
        return DtlError("'id' for a datastore collection should be defined'")
    id = UUID(id)

    tags = json.get("tags")
    if tags is None:
        return DtlError("'tags' for a datastore collection should be defined'")

    # TODO? the `create_datastore_collection` does not return the storeIds

    # Todo?
    # the create function return key "stores"
    stores = None
    storeIds = None
    if "stores" in json.keys():
        stores = _parse_list(_datastore_from_payload)(json.get("stores"))
    if "storeIds" in json.keys():
        storeIds = json.get("storeIds")
    if storeIds is None and stores is not None and len(stores) > 0:
        storeIds = list(map(lambda datastore: datastore.id, stores))

    description = json.get("description")
    ontology_ids = json.get("ontologyIds")

    datastore_collection = DatastoreCollection(name, storeIds, stores, description, tags, ontology_ids, id)

    return datastore_collection
